/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions.annots.form;

import android.graphics.RectF;

import com.foxit.sdk.PDFException;
import com.foxit.sdk.PDFViewCtrl;
import com.foxit.sdk.common.fxcrt.PointF;
import com.foxit.sdk.pdf.PDFPage;
import com.foxit.sdk.pdf.actions.Action;
import com.foxit.sdk.pdf.interform.Control;
import com.foxit.sdk.pdf.interform.FillerAssistCallback;
import com.foxit.sdk.pdf.interform.TimerCallback;
import com.foxit.uiextensions.DocumentManager;
import com.foxit.uiextensions.Module;
import com.foxit.uiextensions.UIExtensionsManager;
import com.foxit.uiextensions.annots.link.LinkModule;
import com.foxit.uiextensions.utils.AppDmUtil;
import com.foxit.uiextensions.utils.AppUtil;

public class FormFillerAssistImpl extends FillerAssistCallback {

    private PDFViewCtrl mPDFViewCtrl;
    protected boolean bWillClose = false;
    boolean bWillSave = false;
    private boolean isScaling = false;
    private FormFillerAnnotHandler.FillerFocusEventListener mFocusEventListener;
    private FormFillerAnnotHandler.FillerPopupMenuListener mPopupMenuListener;

    public FormFillerAssistImpl(PDFViewCtrl pdfViewCtrl, FormFillerAnnotHandler.FillerFocusEventListener focusEventListener) {
        this.mPDFViewCtrl = pdfViewCtrl;
        mFocusEventListener = focusEventListener;
    }

    public void setScaling(boolean scaling) {
        isScaling = scaling;
    }

    void resetRefreshRect() {
        mLastRefreshPdfRect.setEmpty();
    }

    @Override
    public void focusGotOnControl(Control control, String filedValue) {
        if (mFocusEventListener != null)
            mFocusEventListener.focusGotOnControl(control, filedValue);
    }

    @Override
    public void focusLostFromControl(Control control, String filedValue) {
        if (mFocusEventListener != null)
            mFocusEventListener.focusLostFromControl(control, filedValue);
    }

    private RectF mLastRefreshPdfRect = new RectF();

    @Override
    public void refresh(PDFPage page, com.foxit.sdk.common.fxcrt.RectF rect) {
        if (bWillClose || bWillSave || isScaling) return;
        try {
            if (!mLastRefreshPdfRect.equals(AppUtil.toRectF(rect))) {
                int pageIndex = page.getIndex();
                if (!mPDFViewCtrl.isPageVisible(pageIndex)) return;
                RectF viewRect = new RectF(0, 0, mPDFViewCtrl.getDisplayViewWidth(), mPDFViewCtrl.getDisplayViewHeight());
                RectF pdfRect = AppUtil.toRectF(rect);
                mPDFViewCtrl.convertPdfRectToPageViewRect(pdfRect, pdfRect, pageIndex);
                RectF _rect = new RectF(pdfRect);
                mPDFViewCtrl.convertPageViewRectToDisplayViewRect(pdfRect, pdfRect, pageIndex);
                if (!viewRect.intersect(pdfRect)) return;
                _rect.inset(-5, -5);
                mPDFViewCtrl.refresh(pageIndex, AppDmUtil.rectFToRect(_rect));
            }
            mLastRefreshPdfRect = AppUtil.toRectF(rect);
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void release() {
    }

    @Override
    public boolean setTimerCallback(int elapse, TimerCallback timer, Integer out_timer_id) {
        return false;
    }

    @Override
    public boolean killTimer(int timer_id) {
        return false;
    }

    @Override
    public Object createPopupMenu() {
        return new Object();
    }

    @Override
    public boolean appendPopupMenuItem(Object h_popup_menu, int popup_menu_item, String default_string) {
        if (mPopupMenuListener == null) return false;
        mPopupMenuListener.appendPopupMenuItem(popup_menu_item, default_string);
        return false;
    }

    @Override
    public boolean enablePopupMenuItem(Object h_popup_menu, int popup_menu_item, boolean is_enabled) {
        if (mPopupMenuListener == null) return false;
        mPopupMenuListener.enablePopupMenuItem(popup_menu_item, is_enabled);
        return false;
    }

    @Override
    public boolean showPopupMenu(Object h_popup_menu, PointF point) {
        if (mPopupMenuListener == null) return false;
        mPopupMenuListener.showPopupMenu(point);
        return false;
    }

    @Override
    public void destroyPopupMenu(Object h_popup_menu) {
    }

    @Override
    public void setClipboardText(String text) {
        if (mPopupMenuListener == null) return;
        mPopupMenuListener.setClipboardText(text);
    }

    @Override
    public String getClipboardText() {
        if (mPopupMenuListener == null) return null;
        return mPopupMenuListener.getClipboardText();
    }

    @Override
    public void reportInvalidValue(String form_field_name, String invalid_value) {
    }

//    @Override //todo merge code
//    public void executeAction(PDFPage page, Action action) {
//        // MOBRD-7847
//        // 需要处理除FPD_ActionType:Hide/Named/SubmitForm/ResetForm/URI 之外的类型
//        LinkModule linkModule = (LinkModule) ((UIExtensionsManager)mPDFViewCtrl.getUIExtensionsManager()).getModuleByName(Module.MODULE_NAME_LINK);
//        if (linkModule != null) {
//            DocumentManager documentManager = ((UIExtensionsManager) mPDFViewCtrl.getUIExtensionsManager()).getDocumentManager();
//            if (documentManager.getCurrentAnnot() != null) {
//                documentManager.setCurrentAnnot(null, false);
//            }
//            linkModule.jumpToActionDest(action);
//        }
//    }

    public void setPopupMenuListener(FormFillerAnnotHandler.FillerPopupMenuListener listener) {
        mPopupMenuListener = listener;
    }

    public void destroy(){
        mFocusEventListener = null;
        mPopupMenuListener = null;
    }
}
